// thumbnail.h
// thumbnail und thumbnail-liste

#ifndef _H_THUMBNAIL
#define _H_THUMBNAIL 1


#include <qimage.h> 

class Thumbnail
{
   public:
   QImage * bild;
   char   * name;
   bool   selektiert;
   
   Thumbnail();
   ~Thumbnail();
   
   void setzeNamen(char *string);
   void setzeBild(QImage * einBild);
   
};

class ThumbnailListe
{
   private:
   Thumbnail ** liste;
   int anzahl;
   int platzInListe;

   public:
   ThumbnailListe();
   ~ThumbnailListe();
   
  void add(Thumbnail *neu);
  void loescheAlle();
  void holSelektiertListe(int *anzahl,int **selektiertListe);
  
  int anzahlBilder();
  Thumbnail * gibBild(int nummer );
    
};

#endif
